# -*- coding: utf-8 -*-
#
# Copyright (C) 2011 Roberto Bordolanghi
# 
# This file is part of the Screenshot Paste plugin for Trac.
# 
# The Screenshot Paste plugin for Trac is free software: you can 
# redistribute it and/or modify it under the terms of the GNU General 
# Public License as published by the Free Software Foundation, either 
# version 3 of the License, or (at your option) any later version.
# 
# The Screenshot Paste plugin for Trac is distributed in the hope that 
# it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with the Screenshot Paste plugin for Trac.  
# If not, see <http://www.gnu.org/licenses/>.
#

import base64
import cStringIO
import csv
import os
import pkg_resources
import re
import shutil
import sys
import time
import traceback

from datetime import datetime
from operator import itemgetter, attrgetter
from tempfile import NamedTemporaryFile

from trac.attachment import Attachment
from trac.core import *
from trac.perm import IPermissionRequestor, PermissionError
from trac.resource import Resource
from trac.util import get_reporter_id
from trac.util.datefmt import utc, to_utimestamp
from trac.util.text import CRLF
from trac.web.api import IRequestHandler
from trac.web.chrome import add_notice, add_warning, add_stylesheet
from trac.wiki.model import WikiPage

from screenshotpaste.util import *

try:
    from trac.util.translation import domain_functions
    _, tag_, N_, add_domain = domain_functions('screenshotpaste', ('_', 'tag_', 'N_', 'add_domain'))
except ImportError:
	from trac.util.translation import _, N_
	tag_ = _
	add_domain = lambda env_path, locale_dir: None

class ScreenshotPasteSystem(Component):
    """Screenshot Paste system for Trac."""

    implements(IRequestHandler)


    # IRequestHandler methods
    def match_request(self, req):
        type = req.args.get('type', '')
        
        match = False
        
        if req.path_info.startswith('/uploadscreenshot'):
           match = True
            
        return match

    def process_request(self, req):
        self.env.log.debug("process_request: %s" % req.path_info)
    
        if req.path_info.startswith('/uploadscreenshot') and ('TICKET_CREATE' in req.perm):

            if req.method == 'POST':
                if 'screenshot' in req.args:
                    #self.env.log.debug(req.args['screenshot'].value)
                    self._upload_screenshot(req, req.args['screenshot'].value, req.args['original_path'], req.args['realm'], req.args['id'])

            result = 'OK'
            req.send_header("Content-Length", len(result))
            req.write(result)
            return 

        raise TracError("Unknown option.")

    def _upload_screenshot(self, req, data, original_path, realm, id):
        decoded_data = base64.b64decode(data)

        ts = to_utimestamp(datetime.now(utc))
        
        target_filename = 'tmp_screen'+str(ts)+'.jpg'
        target_filename = os.path.basename(target_filename)

        target_dir = os.path.join(self.env.path, 'tmpscreenshots')
        
        if not os.access(target_dir, os.F_OK):
            os.makedirs(target_dir)
            
        target_path = os.path.join(target_dir, target_filename)
        
        try:
            target_file = open(target_path, 'wb')
            target_file.write(decoded_data)
        finally:
            target_file.close()
        
        parent_realm = Resource(realm)
        parent = parent_realm(id=id)

        filename = 'screen'+str(ts)+'.jpg'
        filename = os.path.basename(filename)
        
        attachment = Attachment(self.env, parent.child('attachment', None))
        
        parent_resource = attachment.resource.parent

        # Update the attachment resource
        attachment.description = req.args.get('description', '')
        attachment.author = get_reporter_id(req, 'author')
        attachment.ipnr = req.remote_addr

        target_file_attachment_len = os.path.getsize(target_path)
        target_file_attachment = open(target_path, 'rb')
        attachment.insert(filename, target_file_attachment, target_file_attachment_len)
